class FailedAssertion(Exception):
    pass


class NotAllowedComparison(Exception):
    pass


class InValidCallAttributeError(Exception):
    pass


class InValidCallAttributeModifierError(Exception):
    pass
